!=================================================================
!=================================================================
!=================================================================
!      =====                                           =====
!      =====           MODULE ConstantsClass           =====
!      =====                                           =====
!=================================================================
!=================================================================
!=================================================================

!=================================================================
! Define and initialize some useful global constants.
!
! Revision history:
!    Date         Programmer                 Description of change
!    ----         ----------                 ---------------------
!    03/20/07     Craig  Mattocks, UNC-IMS   Wrote original code
!=================================================================

!=======================================
! Define and initialize global constants
!=======================================
MODULE ConstantsClass

	!--------------------------------------------
	! Force explicit declaration of all variables
	!--------------------------------------------
	IMPLICIT none

	!--------------------------------------------------------------
	! Retain values of class instance variables between invocations
	!--------------------------------------------------------------
	SAVE

	!----------------------
	! Commonly used numbers
	!----------------------
	REAL*8, PARAMETER :: zero		=  0.d0
	REAL*8, PARAMETER :: half		=  .5d0
	REAL*8, PARAMETER :: one		=  1.d0
	REAL*8, PARAMETER :: sqrt2		=  1.41421356237309504880168872420969807856967187537694807317667973799d0
	REAL*8, PARAMETER :: two		=  2.d0
	REAL*8, PARAMETER :: three		=  3.d0
	REAL*8, PARAMETER :: four		=  4.d0
	REAL*8, PARAMETER :: five		=  5.d0
	REAL*8, PARAMETER :: six		=  6.d0
	REAL*8, PARAMETER :: seven		=  7.d0
	REAL*8, PARAMETER :: eight		=  8.d0
	REAL*8, PARAMETER :: nine		=  9.d0
	REAL*8, PARAMETER :: ten		= 10.d0
	REAL*8, PARAMETER :: twenty	= 20.d0

	REAL*8, PARAMETER :: oneHundredth		=      .01d0
	REAL*8, PARAMETER :: oneTenth				=      .1d0
	REAL*8, PARAMETER :: twoPointFive		=     2.5d0
	REAL*8, PARAMETER :: threePointFive		=     3.5d0
	REAL*8, PARAMETER :: twentyFour			=    24.d0
	REAL*8, PARAMETER :: twentyFive			=    25.d0
	REAL*8, PARAMETER :: seventyFive			=    75.d0
	REAL*8, PARAMETER :: oneHundred			=   100.d0
	REAL*8, PARAMETER :: oneEighty			=   180.d0
	REAL*8, PARAMETER :: threeSixty			=   360.d0
	REAL*8, PARAMETER :: fourHundred			=   400.d0
	REAL*8, PARAMETER :: oneThousand			=  1000.d0
	REAL*8, PARAMETER :: thirtySixHundred	=  3600.d0
	REAL*8, PARAMETER :: tenThousand			= 10000.d0

	!-----------------------
	! Flags for missing data
	!-----------------------
	CHARACTER*8, PARAMETER :: missingString  = "Missing!"
	INTEGER    , PARAMETER :: missingInteger = -9999
	REAL*4     , PARAMETER :: missingFloat   = -9999.
	REAL*8     , PARAMETER :: missingDouble  = -9999.d0
	INTEGER                :: missing

	!---------------------
	! Big and small values
	!---------------------
	REAL*8, PARAMETER :: big   = 1.d+25
	REAL*8, PARAMETER :: small = 1.d-5

	!---------------------
	! pi and factors of pi
	!---------------------
!	REAL*8, PARAMETER :: pi = two * DASIN(one)
	REAL*8, PARAMETER :: pi = 3.141592653589793238462643383279502884197d0
	REAL*8, PARAMETER :: pi2 = 1.57079632679489661923132169163975144209858d0
	REAL*8, PARAMETER :: twopi = 6.283185307179586476925286766559005768394d0 

	!----------------------------------------
	! Degrees <--> radians conversion factors
	!----------------------------------------
	REAL*8, PARAMETER :: deg2rad = pi / oneEighty
	REAL*8, PARAMETER :: rad2deg = oneEighty / pi

	!-------------------
	! Conversion factors
	!-------------------
!	REAL*8, PARAMETER :: ft2m = 0.3048d0              ! Intl   feet --> m
	REAL*8, PARAMETER :: ft2m = 1200.d0 / 3937.d0     ! Survey feet --> m
	REAL*8, PARAMETER :: m2ft = 3937.d0 / 1200.d0     ! m --> survey feet


END MODULE ConstantsClass
