!=================================================================
!=================================================================
!=================================================================
!      =====                                           =====
!      =====             MODULE DebugClass             =====
!      =====                                           =====
!=================================================================
!=================================================================
!=================================================================

!====================================================
! This module provides a global debugging capability.
!====================================================
MODULE DebugClass

	!--------------------------------------------
	! Force explicit declaration of all variables
	!--------------------------------------------
	IMPLICIT none

	!-----------------------------------------------------------------
	! By default, encapsulate (hide) all class variables and functions
	!-----------------------------------------------------------------
	PRIVATE

	!--------------------------------------
	! Expose these class instance variables
	!--------------------------------------
	PUBLIC :: OFF, ON
	PUBLIC :: INFO, WARN, ERROR, FATAL

	!-----------------------------
	! Expose these class functions
	!-----------------------------
	PUBLIC :: setDebug, isDebugOn
	PUBLIC :: setDebugLevel, getDebugLevel

	!-------------
	! Debug switch
	!-------------
	LOGICAL, PARAMETER :: OFF	= .FALSE.
	LOGICAL, PARAMETER :: ON	= .TRUE.
	LOGICAL, STATIC :: debug = ON					! Default switch setting

	!-------------------------------
	! Debug severity (filter) levels
	!-------------------------------
	INTEGER, PARAMETER :: INFO		= 1
	INTEGER, PARAMETER :: WARN		= 2
	INTEGER, PARAMETER :: ERROR	= 3
	INTEGER, PARAMETER :: FATAL	= 4
	INTEGER, STATIC :: level = WARN				! Default filter level

	CONTAINS

		!---------------------------
		! Setter (mutator) functions
		!---------------------------
		SUBROUTINE setDebug(onOff)
			LOGICAL, INTENT(IN) :: onOff
			debug = onOff
		END SUBROUTINE setDebug

		SUBROUTINE setDebugLevel(lvl)
			INTEGER, INTENT(IN) :: lvl
			IF (lvl < INFO) THEN
				level = INFO
			ELSE IF (lvl > FATAL) THEN
				level = FATAL
			ELSE
				level = lvl
			END IF
		END SUBROUTINE setDebugLevel

		!----------------------------
		! Getter (accessor) functions
		!----------------------------
		LOGICAL FUNCTION isDebugOn()
			isDebugOn = debug
		END FUNCTION isDebugOn

		INTEGER FUNCTION getDebugLevel()
			getDebugLevel = level
		END FUNCTION getDebugLevel

END MODULE DebugClass
