!=================================================================
!=================================================================
!=================================================================
!		 =====														 =====
!		 =====				 MODULE TimerClass				 =====
!		 =====														 =====
!=================================================================
!=================================================================
!=================================================================

!========================================================
! This module provides a CPU time measurement capability.
!
! Useage: USE TimerClass	! for access to data
!		  CALL tic			! start clock
!		  ...				! Use some cpu time
!		  cputime = toc ()	! for increment in seconds
!========================================================
MODULE TimerClass

		!--------------------------------------------
		! Force explicit declaration of all variables
		!--------------------------------------------
		IMPLICIT none

		!--------------------------------------------------------------
		! Retain values of class instance variables between invocations
		!--------------------------------------------------------------
		SAVE

		!---------------------------------------
		! Define and initialize global constants
		!---------------------------------------
		REAL, PRIVATE :: before		! current value of system clock
		REAL, PRIVATE :: after		! ending	 value of system clock

		CONTAINS

			! -----------------------------------------------
			! Model the matlab tic function, for use with toc
			! -----------------------------------------------
			SUBROUTINE tic 
				CALL cpu_time (before)
			END SUBROUTINE tic

			! -----------------------------------------------
			! Model the matlab toc function, for use with tic
			! -----------------------------------------------
			REAL FUNCTION toc ( ) 
				CALL cpu_time (after)		! Start CPU time "stopwatch"
				toc = after - before
			END FUNCTION toc

END MODULE TimerClass
