#!/usr/bin/perl -w
#
die " -- Not enough arguments
    Usage: hbl_to_ww3.pl inputDir outputFile \n"
    if @ARGV < 2;

$inputDir = $ARGV[0];
$outputFile = $ARGV[1];

# we want to process each snp file in the input dir.
opendir(DIR, $inputDir);
@files = grep(/\.snp$/,sort readdir(DIR));
closedir(DIR);

my @counts = count_lat_long("$inputDir/$files[0]");
foreach $file (@files) {
   print "Processing $file\n";
   write_the_time_header($file, $outputFile);
   write_the_wind_data("$inputDir/$file", $outputFile, $counts[0], $counts[1]);
}

exit( 0 );


# ============================================================================
#
#   Subroutine: calculate the number of lat-long variables in a file.
#   Returns an array with the lat and long dimensions.
# ----------------------------------------------------------------------------

sub count_lat_long {
   my ($inputFile) = @_;
   my @returnArray;
   my @fields;
   my $lon = "";
   my $latCount;
   my $lonCount = 0;

    open FIN, "<$inputFile"
        or die " -- Cannot open file $inputFile: $!";

    my $i = 0;

    while( <FIN> ) {
        chomp;
        @fields = split /\s+/, $_;
        if ($lon ne "" && $fields[1] ne $lon) {
           # The lon value has changed, but we have set it at least once.
           if ($lonCount == 0) {
              # We only need to do this once
              $lonCount = $i;
           }
        } else {
           $lon = $fields[1];
        }
        $i++;
    }

   $latCount = $i/$lonCount;

   print "latCount: $latCount\n";
   print "lonCount: $lonCount\n";
   $returnArray[0] = $latCount;
   $returnArray[1] = $lonCount;

   return @returnArray;
}

# ============================================================================
#
#   Subroutine: write a time header for every wind field.
# ----------------------------------------------------------------------------

sub write_the_time_header {

    my( $inputFile, $file ) = @_;

    # The input file format is stormname_MM_dd_yy_hh_mm.snp
    # where the storm name can itself contain underscores.  Currently the 
    # year is 2 digit, but we code it up to (hopefully) handle a 4 digit value.

    $inputFile=~ s/\..*//;
    @fileFields = split /_/, $inputFile;
    $lastIndex = $#fileFields;

    # We want to write the data as yyyyMMdd.hhmmss
    my $year = $fileFields[$lastIndex - 2];
    if ($year < 100) {
       # a 2 digit year
       if ($year < 10) {
          $year += 2000;
       } else {
          $year += 1900;
       }
    }

    my $month = $fileFields[$lastIndex - 4];
    my $days = $fileFields[$lastIndex - 3];
    my $hours = $fileFields[$lastIndex - 1];
    my $minutes = $fileFields[$lastIndex];
    my $seconds = "00";
    

    open FOUT, ">>$file"
        or die " -- Cannot open file $file: $!";

    $| = 1;

    my $time = "$year$month$days.$hours$minutes$seconds";
    print FOUT "$time\n";

    close FOUT;
}

# ============================================================================
#
#   Subroutine: append the wind data from the input file to the output file.
# ----------------------------------------------------------------------------

sub write_the_wind_data {

    # We are going to read the 10 minute u and 
    my( $inputFile, $outputFile, $latCount, $lonCount) = @_;

    open FOUT, ">>$outputFile"
        or die " -- Cannot open file $outputFile: $!";

    open FIN, "<$inputFile"
        or die " -- Cannot open file $inputFile: $!";

    $| = 1;

    my $i = 0;
    my $j = 0;
    my @fields;

    while( <FIN> ) {
        chomp;
        @fields = split /\s+/, $_;
        printf FOUT "%8.3f", $fields[6];
        $i++;
        $j++;
        if (!($i % 10)) {
           printf FOUT "\n";
           $i = 0;
        } else {
           if (!($j % $lonCount)) {
              printf FOUT "\n";
              $i = 0;
           }
        }
    }
    close FIN;

    $i = 0;
    $j = 0;
    open FIN, "<$inputFile";
    while( <FIN> ) {
        chomp;
        @fields = split /\s+/, $_;
        printf FOUT "%8.3f", $fields[7];
        $i++;
        $j++;
        if (!($i % 10)) {
           printf FOUT "\n";
           $i = 0;
        } else {
           if (!($j % $lonCount)) {
              printf FOUT "\n";
              $i = 0;
           }
        }

    }

    close FIN;
    close FOUT;
}

# ============================================================================
